function [ x ] = broyden( F, x0, Tol, Nmax, varnames )
% BROYDEN Para aproximar una solucin de un sistema de ecuaciones no lineales
% escrito en notacin vectorial F(x)=0, a partir de una aproximacin inicial
% x0.
%
% Parmetros:
%        F: Funcin de R^n->R^n para la que se desea encontrar una solucin;
%           dada como una matriz de celdas de 1 x n, donde las celdas son ca-
%           denas de caracteres que representan las n funciones coordenadas.
%       x0: Aproximacin inicial, como un vector de 1 x n.
%      Tol: Error absoluto mnimo entre aproximaciones sucesivas.
%     Nmax: Nmero mnimo de iteraciones a realizar antes de obtener una apro-
%           ximacin o fallar.
% varnames: Caracteres de las variables para las funciones coordenadas de F, 
%           en un vector de 1 x n.
%
%        x: Solucin aproximada a F, o mensaje de error.

varnamescell = num2cell(varnames);
xcell = num2cell(x0);
[~, n] = size(x0);
x = x0';

% Clculo de J(x0)
J = [];
for j=1:n
    for k=1:n
        fjdiffk = inline(char(diff(sym(F(j)),varnames(k))),varnamescell{:});
        J(j,k) = fjdiffk(xcell{:});
    end
end
A0 = J;

% Clculo de F(x0)
v = [];
for j=1:n
    fj = inline(char(F(j)),varnamescell{:});
    v(j) = fj(xcell{:});
end
v = v';

A = inv(A0);

s = -A*v;
x = x + s;
k = 2;

while(k <= Nmax)
    
    xcell = num2cell(x);
    
    w = v;
    for j=1:n
        fj = inline(char(F(j)),varnamescell{:});
        v(j) = fj(xcell{:});
    end
    y = v - w;
    
    z = -A*y;
    
    p = -(s')*z;
    
    u = (s'*A)';
    
    A = A + (1/p)*(s+z)*u';
    
    s = -A*v;
    
    x = x + s;
    
    norm_infty = norm(s,inf);
    
    if(norm_infty < Tol)
        return
    end
    
    k = k + 1;
end

error(['El mtodo fracas despus de %.d iteraciones. '...
       'Procedimiento terminado sin xito.'],Nmax);
end